import datetime as dt


def test_get_last_day_of_month_newyearseve(sample_store):

    # GIVEN

    # WHEN calling get_until_date
    day = dt.datetime(2001, 12, 31)
    until_date = sample_store.get_until_date(day.year)

    # THEN return should be last december of that year
    assert day.year == until_date.year
    assert 12 == until_date.month
    assert 31 == until_date.day


def test_get_last_day_of_month_1stjanuary(sample_store):

    # GIVEN

    # WHEN calling get_last_day_of_month with 2001-01-01
    day = dt.datetime(2001, 1, 1)
    until_date = sample_store.get_until_date(day.year)

    # THEN return should be 2001-01-31
    assert day.year == until_date.year
    assert 12 == until_date.month
    assert 31 == until_date.day



"""
    conftest for genotype API
"""

import pytest
from cg.apps.gt import GenotypeAPI

CONFIG = {
    "genotype": {
        "database": "database",
        "config_path": "config/path",
        "binary_path": "gtdb",
    }
}


@pytest.fixture(scope="function")
def genotype_config():
    """
    genotype config fixture
    """

    _config = CONFIG

    return _config


@pytest.fixture(scope="function")
def genotypeapi():
    """
    genotype API fixture
    """

    _genotype_api = GenotypeAPI(CONFIG)
    return _genotype_api



"""Fixtures for meta/upload tests"""

import json

import pytest

from cg.apps.coverage.api import ChanjoAPI
from cg.meta.upload.coverage import UploadCoverageApi
from cg.meta.upload.mutacc import UploadToMutaccAPI
from cg.meta.upload.observations import UploadObservationsAPI
from cg.meta.upload.scoutapi import UploadScoutAPI


class MockAnalysis:
    """Mock an analysis object"""

    @staticmethod
    def get_latest_metadata(family_id=None):
        """Mock get_latest_metadata"""
        # Returns: dict: parsed data
        # Define output dict
        outdata = {
            "analysis_sex": {"ADM1": "female", "ADM2": "female", "ADM3": "female"},
            "family": family_id or "yellowhog",
            "duplicates": {"ADM1": 13.525, "ADM2": 12.525, "ADM3": 14.525},
            "genome_build": "hg19",
            "rank_model_version": "1.18",
            "mapped_reads": {"ADM1": 98.8, "ADM2": 99.8, "ADM3": 97.8},
            "mip_version": "v4.0.20",
            "sample_ids": ["2018-20203", "2018-20204"],
            "sv_rank_model_version": "1.08",
        }
        return outdata

    @staticmethod
    def convert_panels(customer_id, panels):
        """Mock convert_panels"""
        _ = customer_id, panels
        return ""


class MockCoverage(ChanjoAPI):
    """Mock chanjo coverage api"""


class MockMutaccAuto:
    """ Mock class for mutacc_auto api """

    @staticmethod
    def extract_reads(*args, **kwargs):
        """mock extract_reads method"""
        _, _ = args, kwargs

    @staticmethod
    def import_reads(*args, **kwargs):
        """ mock import_reads method """
        _, _ = args, kwargs


class MockScoutApi:
    """Mock class for Scout api"""

    @staticmethod
    def get_causative_variants(case_id):
        """mock get_causative_variants"""
        _ = case_id
        return []

    @property
    def client(self):
        """Client URI"""
        return "mongodb://"

    @property
    def name(self):
        """Name property"""
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @name.deleter
    def name(self):
        del self._name


class MockLoqusAPI:
    """ Mock LoqusAPI class"""

    def __init__(self, analysis_type="wgs"):
        self.analysis_type = analysis_type

    @staticmethod
    def load(*args, **kwargs):
        """ Mock load method"""
        _ = args
        _ = kwargs
        return dict(variants=12)

    @staticmethod
    def get_case(*args, **kwargs):
        """ Mock get_case method"""
        _ = args
        _ = kwargs
        return {"case_id": "case_id", "_id": "123"}

    @staticmethod
    def get_duplicate(*args, **kwargs):
        """Mock get_duplicate method"""
        _ = args
        _ = kwargs
        return {"case_id": "case_id"}


class MockLims:
    """ Mock Lims API """

    lims = None

    def __init__(self, samples):
        self.lims = self
        self._samples = samples

    def sample(self, sample_id):
        """ Returns a lims sample matching the provided sample_id """
        for sample in self._samples:
            if sample["id"] == sample_id:
                return sample
        return None


@pytest.fixture(name="lims_family")
def fixture_lims_family():
    """ Returns a lims-like family of samples """
    return json.load(open("tests/fixtures/report/lims_family.json"))


@pytest.fixture(name="lims_samples")
def fixture_lims_samples(lims_family):
    """ Returns the samples of a lims family """
    return lims_family["samples"]


@pytest.yield_fixture(scope="function")
def upload_observations_api(analysis_store, populated_housekeeper_api):
    """ Create mocked UploadObservationsAPI object"""

    loqus_mock = MockLoqusAPI()

    _api = UploadObservationsAPI(
        status_api=analysis_store,
        hk_api=populated_housekeeper_api,
        loqus_api=loqus_mock,
    )

    yield _api


@pytest.yield_fixture(scope="function")
def upload_observations_api_wes(analysis_store, populated_housekeeper_api):
    """ Create mocked UploadObservationsAPI object"""

    loqus_mock = MockLoqusAPI(analysis_type="wes")

    _api = UploadObservationsAPI(
        status_api=analysis_store,
        hk_api=populated_housekeeper_api,
        loqus_api=loqus_mock,
    )

    yield _api


@pytest.yield_fixture(scope="function")
def upload_scout_api(scout_api, madeline_api, lims_samples, populated_housekeeper_api):
    """Fixture for upload_scout_api"""
    analysis_mock = MockAnalysis()
    lims_api = MockLims(lims_samples)

    _api = UploadScoutAPI(
        hk_api=populated_housekeeper_api,
        scout_api=scout_api,
        madeline_api=madeline_api,
        analysis_api=analysis_mock,
        lims_api=lims_api,
    )

    yield _api


@pytest.yield_fixture(scope="function")
def mutacc_upload_api():
    """
    Fixture for a mutacc upload api
    """

    scout_api = MockScoutApi()
    mutacc_auto_api = MockMutaccAuto()

    _api = UploadToMutaccAPI(scout_api=scout_api, mutacc_auto_api=mutacc_auto_api)

    return _api


@pytest.yield_fixture(scope="function")
def coverage_upload_api(chanjo_config_dict, populated_housekeeper_api):
    """Fixture for coverage upload API"""
    hk_api = populated_housekeeper_api
    status_api = None
    coverage_api = MockCoverage(chanjo_config_dict)
    _api = UploadCoverageApi(status_api=status_api, hk_api=hk_api, chanjo_api=coverage_api)
    return _api


@pytest.yield_fixture(scope="function")
def analysis(analysis_store, case_id):
    """Fixture to mock an analysis"""
    _analysis = analysis_store.add_analysis(pipeline="pipeline", version="version")
    _analysis.family = analysis_store.family(case_id)
    _analysis.config_path = "dummy_path"
    yield _analysis



import datetime as dt
from itertools import groupby
from sqlalchemy import func

from cg.store import models
from cg.store.api.base import BaseHandler

MONTHS = {
    1: "January",
    2: "February",
    3: "March",
    4: "April",
    5: "May",
    6: "June",
    7: "July",
    8: "August",
    9: "September",
    10: "October",
    11: "November",
    12: "December",
}


class TrendsHandler(BaseHandler):
    """Deprecated in favour of Vogue"""

    @staticmethod
    def get_last_day_of_previous_year(year: int):
        return TrendsHandler.get_last_day_of_year(year - 1)

    @staticmethod
    def get_last_day_of_year(year):
        return dt.date(year, 12, 31)

    @staticmethod
    def get_from_date(year):
        return TrendsHandler.get_last_day_of_previous_year(int(year))

    @staticmethod
    def get_until_date(year):
        return TrendsHandler.get_last_day_of_year(int(year))

    def samples_per_month(self, year):
        """Fetch samples per month. Grouped by priority."""
        query = (
            self.session.query(
                models.Customer.priority.label("priority"),
                func.month(models.Sample.received_at).label("month_no"),
                func.count(models.Sample.id).label("count"),
            )
            .join(models.Sample.customer)
            .filter(
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
            )
            .group_by("priority", "month_no")
        )

        for cust_priority, results in groupby(query, key=lambda result: result.priority):
            counts = {MONTHS[result.month_no]: result.count for result in results}
            data = {
                "name": cust_priority,
                "results": {month: counts.get(month) or None for month in MONTHS.values()},
            }
            yield data

    def samples_per_month_application(self, year):
        """Fetch samples per month. Grouped by application cathegory."""

        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                func.count(models.Sample.id).label("count"),
            )
            .join(
                models.Sample.application_version,
                models.ApplicationVersion.application,
            )
            .filter(
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
                models.Sample.received_at,
                models.Application.category,
            )
            .group_by("category", "month_no")
        )

        for application_cathegory, results in groupby(query, key=lambda result: result.category):
            counts = {MONTHS[result.month_no]: result.count for result in results}
            data = {
                "name": application_cathegory,
                "results": {month: counts.get(month) or None for month in MONTHS.values()},
            }
            yield data

    def received_to_delivered(self, year):
        """Calculate averages from received to delivered."""
        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                (
                    func.avg(
                        func.datediff(models.Sample.delivered_at, models.Sample.received_at)
                    ).label("average")
                ),
            )
            .join(
                models.Sample.customer,
                models.Sample.application_version,
                models.ApplicationVersion.application,
            )
            .filter(
                models.Customer.priority == "diagnostic",
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
                models.Sample.delivered_at is not None,
            )
            .group_by(
                models.Application.category,
                func.month(models.Sample.received_at),
            )
        )
        for category, results in groupby(query, key=lambda result: result.category):
            averages = {
                MONTHS[result.month_no]: float(result.average) if result.average else None
                for result in results
            }
            yield {
                "name": category,
                "results": {month: averages.get(month) or None for month in MONTHS.values()},
            }

    def received_to_prepped(self, year):
        """Calculate averages to prepp samples."""
        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                (
                    func.avg(
                        func.datediff(models.Sample.prepared_at, models.Sample.received_at)
                    ).label("average")
                ),
            )
            .join(
                models.Sample.customer,
                models.Sample.application_version,
                models.ApplicationVersion.application,
            )
            .filter(
                models.Customer.priority == "diagnostic",
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
            )
            .group_by(
                models.Application.category,
                func.month(models.Sample.received_at),
            )
        )
        for category, results in groupby(query, key=lambda result: result.category):
            averages = {
                MONTHS[result.month_no]: float(result.average) if result.average else None
                for result in results
            }
            yield {
                "name": category,
                "results": {month: averages.get(month) or None for month in MONTHS.values()},
            }

    def prepped_to_sequenced(self, year):
        """Calculate average to sequence samples."""
        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                func.avg(
                    func.datediff(models.Sample.sequenced_at, models.Sample.prepared_at)
                ).label("average"),
            )
            .join(
                models.Sample.customer,
                models.Sample.application_version,
                models.ApplicationVersion.application,
            )
            .filter(
                models.Customer.priority == "diagnostic",
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
            )
            .group_by(
                models.Application.category,
                func.month(models.Sample.received_at),
            )
        )
        for category, results in groupby(query, key=lambda result: result.category):
            averages = {
                MONTHS[result.month_no]: float(result.average) if result.average else None
                for result in results
            }
            yield {
                "name": category,
                "results": {month: averages.get(month) or None for month in MONTHS.values()},
            }

    def sequenced_to_delivered(self, year):
        """Calculate average to deliver samples."""
        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                func.avg(
                    func.datediff(models.Sample.delivered_at, models.Sample.sequenced_at)
                ).label("average"),
            )
            .join(
                models.Sample.customer,
                models.Sample.application_version,
                models.ApplicationVersion.application,
            )
            .filter(
                models.Customer.priority == "diagnostic",
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
                models.Sample.delivered_at is not None,
            )
            .group_by(
                models.Application.category,
                func.month(models.Sample.received_at),
            )
        )
        for category, results in groupby(query, key=lambda result: result.category):
            averages = {
                MONTHS[result.month_no]: float(result.average) if result.average else None
                for result in results
            }
            yield {
                "name": category,
                "results": {month: averages.get(month) or None for month in MONTHS.values()},
            }

    def delivered_to_invoiced(self, year):
        """Calculate average time to invoice samples."""
        query = (
            self.session.query(
                models.Application.category.label("category"),
                func.month(models.Sample.received_at).label("month_no"),
                func.avg(
                    func.datediff(models.Invoice.invoiced_at, models.Sample.delivered_at)
                ).label("average"),
            )
            .join(
                models.Sample.customer,
                models.Sample.application_version,
                models.ApplicationVersion.application,
                models.Sample.invoice,
            )
            .filter(
                models.Customer.priority == "diagnostic",
                models.Sample.received_at > self.get_from_date(year),
                models.Sample.received_at < self.get_until_date(year),
                models.Sample.delivered_at is not None,
                models.Sample.invoice is not None,
                models.Sample.invoice_id is not None,
                models.Invoice.invoiced_at is not None,
            )
            .group_by(
                models.Application.category,
                func.month(models.Sample.received_at),
            )
        )
        for category, results in groupby(query, key=lambda result: result.category):
            averages = {
                MONTHS[result.month_no]: float(result.average) if result.average else None
                for result in results
            }
            yield {
                "name": category,
                "results": {month: averages.get(month) or None for month in MONTHS.values()},
            }



"""Test methods for cg/cli/set/microbial_sample"""
from datetime import datetime

from cg.cli.set import microbial_sample
from cg.store import Store

SUCCESS = 0


def test_invalid_sample_empty_db(cli_runner, base_context):
    # GIVEN an empty database

    # WHEN running set with an sample that does not exist
    sample_id = "dummy_sample_id"
    result = cli_runner.invoke(microbial_sample, [sample_id, "sign"], obj=base_context)

    # THEN then it should complain on invalid sample
    assert result.exit_code != SUCCESS


def test_invalid_sample_non_empty_db(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a non empty database
    helpers.add_microbial_sample_and_order(base_store)

    # WHEN running set with an sample that does not exist
    sample_id = "dummy_sample_id"
    result = cli_runner.invoke(microbial_sample, [sample_id, "sign"], obj=base_context)

    # THEN then it should complain on invalid sample
    assert result.exit_code != SUCCESS


def test_valid_sample_no_apptag_option(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a non empty database
    sample = helpers.add_microbial_sample_and_order(base_store)

    # WHEN running set with an sample but without any options
    sample_id = sample.internal_id
    result = cli_runner.invoke(microbial_sample, [sample_id, "sign"], obj=base_context)

    # THEN then it should complain on missing options
    assert result.exit_code != SUCCESS


def test_invalid_application(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a database with a sample
    sample = helpers.add_microbial_sample_and_order(base_store)
    application_tag = "dummy_application"
    assert (
        base_store.MicrobialSample.query.first().application_version.application.tag
        != application_tag
    )

    # WHEN calling set sample with an invalid application
    result = cli_runner.invoke(
        microbial_sample,
        [sample.internal_id, "sign", "-a", application_tag],
        obj=base_context,
    )

    # THEN then it should complain about missing application instead of setting the value
    assert result.exit_code != SUCCESS
    assert (
        base_store.MicrobialSample.query.first().application_version.application.tag
        != application_tag
    )


def test_valid_application(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a database with a sample and two applications
    sample = helpers.add_microbial_sample_and_order(base_store)
    application_tag = helpers.ensure_application_version(
        base_store, "another_application"
    ).application.tag
    assert (
        base_store.MicrobialSample.query.first().application_version.application.tag
        != application_tag
    )

    # WHEN calling set sample with an valid application
    signature = "sign"
    result = cli_runner.invoke(
        microbial_sample,
        [sample.internal_id, "sign", "-a", application_tag],
        obj=base_context,
    )

    # THEN then the application should have been set
    assert result.exit_code == SUCCESS
    assert (
        base_store.MicrobialSample.query.first().application_version.application.tag
        == application_tag
    )
    assert signature in base_store.MicrobialSample.query.first().comment


def test_invalid_priority(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a database with a sample
    sample = helpers.add_microbial_sample_and_order(base_store)
    priority = "dummy_priority"
    assert base_store.MicrobialSample.query.first().priority != priority

    # WHEN calling set sample with an invalid priority
    result = cli_runner.invoke(
        microbial_sample, [sample.internal_id, "sign", "-p", priority], obj=base_context
    )

    # THEN then it should complain about bad priority instead of setting the value
    assert result.exit_code != SUCCESS
    assert base_store.MicrobialSample.query.first().priority_human != priority


def test_valid_priority(cli_runner, base_context, base_store: Store, helpers):
    # GIVEN a database with a sample
    sample = helpers.add_microbial_sample_and_order(base_store)
    priority = "priority"
    assert base_store.MicrobialSample.query.first().priority != priority

    # WHEN calling set sample with an valid priority
    signature = "sign"

    result = cli_runner.invoke(
        microbial_sample, [sample.internal_id, "sign", "-p", priority], obj=base_context
    )

    # THEN then the priority should have been set
    assert result.exit_code == SUCCESS
    assert base_store.MicrobialSample.query.first().priority_human == priority
    assert signature in base_store.MicrobialSample.query.first().comment



import logging
from pathlib import Path

import ruamel.yaml

from cg.apps import hk, gt, tb
from cg.store import models, Store

LOG = logging.getLogger(__name__)


class UploadGenotypesAPI(object):
    def __init__(
        self,
        status_api: Store,
        hk_api: hk.HousekeeperAPI,
        tb_api: tb.TrailblazerAPI,
        gt_api: gt.GenotypeAPI,
    ):
        self.status = status_api
        self.hk = hk_api
        self.tb = tb_api
        self.gt = gt_api

    def data(self, analysis_obj: models.Analysis) -> dict:
        """Fetch data about an analysis to load genotypes."""
        analysis_date = analysis_obj.started_at or analysis_obj.completed_at
        hk_version = self.hk.version(analysis_obj.family.internal_id, analysis_date)
        hk_bcf = self.hk.files(version=hk_version.id, tags=["snv-gbcf"]).first()
        if hk_bcf is None:
            LOG.warning("unable to find GBCF for genotype upload")
            return None
        data = {"bcf": hk_bcf.full_path, "samples_sex": {}}
        analysis_sexes = self._analysis_sex(hk_version)
        for link_obj in analysis_obj.family.links:
            data["samples_sex"][link_obj.sample.internal_id] = {
                "pedigree": link_obj.sample.sex,
                "analysis": analysis_sexes[link_obj.sample.internal_id],
            }
        return data

    def _analysis_sex(self, hk_version: hk.models.Version) -> dict:
        """Fetch analysis sex for each sample of an analysis."""
        hk_qcmetrics = self.hk.files(version=hk_version.id, tags=["qcmetrics"]).first()
        with Path(hk_qcmetrics.full_path).open() as in_stream:
            qcmetrics_raw = ruamel.yaml.safe_load(in_stream)
        qcmetrics_data = self.tb.parse_qcmetrics(qcmetrics_raw)
        data = {}
        for sample_data in qcmetrics_data["samples"]:
            data[sample_data["id"]] = sample_data["predicted_sex"]
        return data

    def upload(self, data: dict, replace: bool = False):
        """Upload data about genotypes for a family of samples."""
        self.gt.upload(str(data["bcf"]), data["samples_sex"], force=replace)



"""This script tests the cli methods to create prerequisites and start a mip-dna analysis"""
import logging

from cg.cli.workflow.mip_dna.base import start
from cg.constants import EXIT_SUCCESS


def test_dry(cli_runner, mip_context, tb_api):
    """Test command with --dry option"""

    # GIVEN case-id

    # WHEN dry running
    result = cli_runner.invoke(start, ["--dry-run"], obj=mip_context)

    # THEN command should have accepted the option happily
    assert result.exit_code == EXIT_SUCCESS


def test_dna_case_included(cli_runner, mip_context, dna_case, caplog, tb_api):
    """Test command with a dna case"""

    # GIVEN a case that is ready for MIP DNA analysis
    #   -> has a sample that is sequenced and has an dna-application (non-wts)
    for link in dna_case.links:
        sample = link.sample
        assert sample.sequenced_at
        assert sample.application_version.application.analysis_type not in "wts"
    assert not dna_case.analyses

    # WHEN running command
    with caplog.at_level(logging.INFO):
        result = cli_runner.invoke(start, ["--dry-run"], obj=mip_context)

    # THEN command should have printed the case id
    assert result.exit_code == EXIT_SUCCESS
    case_mentioned = False
    for _, level, message in caplog.record_tuples:
        if dna_case.internal_id in message:
            case_mentioned = True
            assert level == logging.INFO
    assert case_mentioned


def test_rna_case_excluded(cli_runner, mip_context, rna_case, caplog, tb_api):
    """Test command with a rna case"""

    # GIVEN a case that is ready for MIP RNA analysis
    #   -> has a sample that is sequenced and has an rna-application (wts)
    for link in rna_case.links:
        sample = link.sample
        assert sample.sequenced_at
        assert sample.application_version.application.analysis_type in "wts"
    assert not rna_case.analyses

    # WHEN running command
    result = cli_runner.invoke(start, ["--dry-run"], obj=mip_context)

    # THEN command should info about it starting the case but warn about skipping
    assert result.exit_code == EXIT_SUCCESS

    case_mentioned = False
    for _, level, message in caplog.record_tuples:
        if rna_case.internal_id in message:
            case_mentioned = True
            assert level == logging.WARNING
    assert case_mentioned



"""Test MIP get files and build bundle"""
import mock
import pytest

from snapshottest import Snapshot

import cg.meta.store.mip as store_mip
from cg.exc import (
    AnalysisNotFinishedError,
    BundleAlreadyAddedError,
)


@mock.patch("cg.store.Store")
@mock.patch("cg.apps.hk.HousekeeperAPI")
@mock.patch("cg.meta.store.mip.add_analysis")
def test_gather_files_and_bundle_in_hk_bundle_already_added(
    mock_add_analysis, mock_housekeeper, mock_cg_store, config_stream, bundle_data
):
    """
    tests the function gather_files_and_bundle_in_housekeeper
    """
    # GIVEN a MIP RNA analysis config file
    mip_rna_config = config_stream["rna_config_store"]

    # WHEN the bundle has already been added to Housekeeper
    mock_add_analysis.return_value = bundle_data
    mock_housekeeper.add_bundle.return_value = None

    # THEN the BundleAlreadyAddedError exception should be raised
    with pytest.raises(BundleAlreadyAddedError) as exc_info:
        store_mip.gather_files_and_bundle_in_housekeeper(
            mip_rna_config, mock_housekeeper, mock_cg_store
        )

    assert exc_info.value.message == "bundle already added"


@mock.patch("cg.store.Store")
@mock.patch("housekeeper.store.models")
@mock.patch("cg.apps.hk.HousekeeperAPI")
@mock.patch("cg.meta.store.mip.reset_case_action")
@mock.patch("cg.meta.store.mip.add_new_analysis")
@mock.patch("cg.meta.store.mip.add_analysis")
def test_gather_files_and_bundle_in_hk_bundle_new_analysis(
    mock_add_analysis,
    mock_add_new_analysis,
    mock_reset_case_action,
    mock_housekeeper_api,
    mock_housekeeper_store,
    mock_cg_store,
    config_stream,
    bundle_data,
):
    """
    tests the function gather_files_and_bundle_in_housekeeper
    """
    # GIVEN a MIP RNA analysis config file
    mip_rna_config = config_stream["rna_config_store"]

    # WHEN adding a new bundle to Housekeeper
    mock_add_analysis.return_value = bundle_data
    mock_bundle = mock_housekeeper_store.Bundle.return_value
    mock_version = mock_housekeeper_store.Version.return_value
    mock_housekeeper_api.add_bundle.return_value = (mock_bundle, mock_version)
    mock_case = mock_cg_store.Family.return_value
    mock_reset_case_action(mock_case)
    mock_add_new_analysis.return_value = mock_cg_store.Analysis.return_value

    store_mip.gather_files_and_bundle_in_housekeeper(
        mip_rna_config, mock_housekeeper_api, mock_cg_store
    )

    # THEN the bundle and version should be added to Housekeeper
    mock_housekeeper_api.include.assert_called()
    mock_housekeeper_api.include.assert_called_with(mock_version)
    mock_housekeeper_api.add_commit.assert_called()
    mock_housekeeper_api.add_commit.assert_called_with(mock_bundle, mock_version)


@mock.patch("cg.meta.store.mip.build_bundle")
@mock.patch("cg.meta.store.mip.parse_sampleinfo")
@mock.patch("cg.meta.store.mip.parse_config")
def test_add_analysis_finished(
    mock_parse_config,
    mock_parse_sample,
    mock_build_bundle,
    config_stream,
    config_data_rna,
    sampleinfo_data,
    rna_deliverables_raw,
):
    """
    tests the function add_analysis when passing a config file of a finished RNA analysis
    """
    # GIVEN a MIP RNA analysis configuration file
    mip_rna_config = config_stream["rna_config_store"]

    # WHEN gathering information from the MIP RNA analysis to store and the analysis is
    # finished
    mock_parse_config.return_value = config_data_rna
    mock_parse_sample.return_value = sampleinfo_data

    result = store_mip.add_analysis(mip_rna_config)

    # THEN the function add_analysis should call the function build_bundle with the correct
    # parameters and return a new bundle
    mock_build_bundle.assert_called()
    mock_build_bundle.assert_called_with(config_data_rna, sampleinfo_data, rna_deliverables_raw)
    assert result == mock_build_bundle(config_data_rna, sampleinfo_data, rna_deliverables_raw)


@mock.patch("cg.meta.store.mip.parse_sampleinfo")
@mock.patch("cg.meta.store.mip.parse_config")
def test_add_analysis_not_finished(
    mock_parse_config, mock_parse_sample, config_stream, config_data_rna, sampleinfo_data
):
    """
    tests the function add_analysis when passing a config file of an unfinished RNA analysis
    """
    # GIVEN a MIP RNA analysis configuration file

    mip_rna_config = config_stream["rna_config_store"]
    mock_parse_config.return_value = config_data_rna
    sampleinfo_data["is_finished"] = False
    mock_parse_sample.return_value = sampleinfo_data

    # WHEN  gathering information from the MIP RNA analysis to store and the analysis is not
    # finished

    with pytest.raises(AnalysisNotFinishedError) as exc_info:
        store_mip.add_analysis(mip_rna_config)

    # THEN the correct exception should be raised
    assert exc_info.value.message == "analysis not finished"


def test_parse_config(snapshot: Snapshot, config_raw: dict):
    """
    tests the function parse_config against a snapshot
    """
    # GIVEN the raw MIP analysis config file

    # WHEN getting the files used to build the bundle
    mip_rna_parse_config = store_mip.parse_config(config_raw)

    # THEN the result should contain the data to be stored in Housekeeper
    snapshot.assert_match(mip_rna_parse_config)


def test_parse_sampleinfo_data(snapshot: Snapshot, files_raw):
    """
    tests the function parse_sampleinfo_data against a snapshot
    """
    # GIVEN the raw MIP sample info file

    rna_sampleinfo = files_raw["rna_sampleinfo"]

    # WHEN getting the smaple info used to build the bundle
    mip_rna_parse_sampleinfo = store_mip.parse_sampleinfo(rna_sampleinfo)

    # THEN the result should contain the data to be stored in Housekeeper
    snapshot.assert_match(mip_rna_parse_sampleinfo)
